#!/usr/bin/env perl


use strict;
use Getopt::Long;

my $video_set_file;
my $threshold_file;
my $detection_file;
my $mer_archive_listing;
my $event;

GetOptions (
    "vid_set_f=s" => \$video_set_file,
    "threshold_f=s" => \$threshold_file,
    "detection_f=s" => \$detection_file,
    "mer_archive_ls=s" => \$mer_archive_listing,
    "event=s" => \$event,
    );

sub err_exit {
    print "[Error] ".shift;
    exit 1;
}

err_exit("Missing required option -v, aborting!\n") unless $video_set_file;
err_exit("Missing required option -t, aborting!\n") unless $threshold_file;
err_exit("Missing required option -d, aborting!\n") unless $detection_file;
err_exit("Missing required option -e, aborting!\n") unless $event;

my %video_set = ();
err_exit("Video set file doesn't exist, aborting!\n") unless -f $video_set_file;
open VSFILE, "<$video_set_file";
<VSFILE>; #Ignore header
while (my $line = <VSFILE>) {
    next if $line =~ /^$/;
    my ($id,) = split(',', $line);
    $video_set{$id}=1;
}
close VSFILE;

my $threshold;

err_exit("Threshold file doesn't exist, aborting!\n") unless -f $threshold_file;
open TFILE, "<$threshold_file";
err_exit("Threshold file header looks deformed, aborting!\n") unless <TFILE> =~ /EventID,QueryType,PRF,DetectionThresholdScore,DetectionThresholdRank/i;
my ($t_e,undef,undef,$threshold,) = split(',',<TFILE>);
close TFILE;
err_exit ("Event '$t_e' does not match expected $event in Threshold file, aborting!\n") unless $t_e eq $event;

err_exit("Detection file doesn't exist, aborting!\n") unless -f $detection_file;
my %detected = ();
my %positives = ();
open DFILE, "<$detection_file";
err_exit("Detection file header looks deformed, aborting!\n") unless <DFILE> =~ /EventID,QueryType,PRF,VideoID,Score,Rank/i;

my @extra = ();
my $line_num = 2;
while (my $line = <DFILE>) {
    chomp($line);
    next if $line =~ /^$/;
    my ($e,undef,undef,$id,$score,$rank) = split(',',$line);
    err_exit ("Event '$e' does not match expected $event on line $line_num in Detection file, aborting!\n") unless $e eq $event;
    err_exit("Score on line $line_num in Detection file is out of bounds, aborting!\n") unless ($score >= 0 && $score <= 1);
    $positives{$id} = 1 if $score > $threshold;
    $detected{$id} = 1;
    push (@extra, $id) unless $video_set{$id};
    err_exit("Rank '$rank' on line $line_num in Detection file doesn't look like a rank, aborting!\n") unless $rank =~ /[0-9]+/;
    $line_num++;
}
close DFILE;

my @missing = ();
foreach my $id (keys %video_set) {
    push (@missing, $id) unless $detected{$id};
}
print "[Error] ".@extra." extraneous VideoID(s) in Detection file ..\n".join("\n", @extra)."\n" if @extra > 0;
print "[Error] Missing ".@missing." VideoID(s) from Detection file ..\n".join("\n", @missing)."\n" if @missing > 0;
exit 1 if @extra > 0 || @missing > 0;

if ($mer_archive_listing) {
    err_exit("MER archive listing file doesn't exist, aborting!\n") unless -f $mer_archive_listing;

    my %recounted = ();
    my @r_extra_mer = ();
    my @r_extra_other = ();
    open RFILE, "<$mer_archive_listing";
    while (my $line = <RFILE>) {
	chomp($line);
	next if $line =~ /^$/;
	if ($line !~ /.*\.mer\.xml$/) {
	    push (@r_extra_other, $line);
	    next;
	}
	my ($id, $e) = split('\.', $line);
	err_exit ("Event '$e' does not match expected $event for recounting $line, aborting!\n") unless $e eq $event;
	$id="$id";
	$recounted{$id} = 1;
	push (@r_extra_mer, "$id.$event.mer.xml") unless $positives{$id};
    }
    close RFILE;

    my @r_missing = ();
    foreach my $id (keys %positives) {
	push (@r_missing, "$id.$event.mer.xml") unless $recounted{$id};
    }
    
    print "[Error] ".@r_extra_other." Extraneous file(s) ..\n".join("\n", @r_extra_other)."\n" if @r_extra_other > 0;
    print "[Error] ".@r_extra_mer." Extraneous recounting(s) ..\n".join("\n", @r_extra_mer)."\n" if @r_extra_mer > 0;
    print "[Error] Missing ".@r_missing." recounting(s) ..\n".join("\n", @r_missing)."\n" if @r_missing > 0;
    exit 1 if @r_extra_other > 0 || @r_extra_mer > 0 || @r_missing > 0;
}

0;
